local TitanPalsTimer = LibStub("AceTimer-3.0")
-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnLoad()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPalsButton_OnLoad(self)

	self.registry = { 
		id = TITAN_PALS_ID,
		menuText = TITAN_PALS_MENU_TEXT, 
		buttonTextFunction = "TitanPanelPalsButton_GetButtonText", 
		tooltipTitle = TITAN_PALS_TOOLTIP,
		tooltipCustomFunction = TitanPanelPalsButton_SetTooltipText,
		icon = TITANPALS_ARTWORK_PATH.."TitanPals",
		iconWidth = 16,
		category = "Information",
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowOffline = false,
			ShowIgnored = false,
			ShowClass = 1,
			ShowNote = false,
			TitanPalsSync = false,
			LoadMessage = 1,
			NoAlts = 1,
			Debug = false,
		}
	};
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	-- For TitanPals Settings Database
	if ( not TitanPalsVars or TitanPalsVars == nil ) then 
		TitanPalsVars = {};
		TitanPalsVars["Version"] = TITAN_PALS_ADDVER;
	end
end

-- **************************************************************************
-- NAME : TitanPanelPals_Init()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPals_Init(TPlayer, TRealm, TFaction)
	SlashCmdList["Pals"] = TitanPalsCommand_Handler;
	SLASH_Pals1 = "/Pals";
	local PalsTimer = TitanGetVar(TITAN_PALS_ID, "TitanPalsTimer");
	-- For Pals Database
	if ( not TitanPalsList or TitanPalsList == nil) then TitanPalsList = {}; end
	if ( not TitanPalsList[TFaction] ) then TitanPalsList[TFaction] = {}; end
	if ( not TitanPalsList[TFaction][TRealm] ) then TitanPalsList[TFaction][TRealm] = {}; end
	if ( not TitanPalsList[TFaction][TRealm][TPlayer] ) then TitanPalsList[TFaction][TRealm][TPlayer] = {}; end
	-- For Alts Database
	if ( not TitanPalsAlts or TitanPalsAlts == nil ) then TitanPalsAlts = {}; end
	if ( not TitanPalsAlts[TFaction] ) then TitanPalsAlts[TFaction] = {}; end
	if ( not TitanPalsAlts[TFaction][TRealm] ) then TitanPalsAlts[TFaction][TRealm] = {}; end
	if ( not TitanPalsAlts[TFaction][TRealm][TPlayer] ) then TitanPalsAlts[TFaction][TRealm][TPlayer] = TPlayer; end

	-- Addon Loading Message
	if DEFAULT_CHAT_FRAME and TitanGetVar(TITAN_PALS_ID, "LoadMessage") then DEFAULT_CHAT_FRAME:AddMessage("Titan Panel [|cffeda55fPals|r] v"..TITAN_PALS_VERSION.." loaded") end

	TitanPalsTimer.ScheduleRepeatingTimer("TitanPalsTimer", TitanPalsTimer_Handler, TITAN_PALS_TIMER)
end

-- **************************************************************************
-- NAME : TitanPanelPals_SyncAlts()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPals_SyncAlts()
	-- Alts to Pals List Syncing
	local TitanPalsFaction = UnitFactionGroup("player");
	local TitanPalsRealm = GetCVar("realmName");
	local TitanPalsRealmUnit = GetUnitName("player");
	local NumPals = GetNumFriends();
	local AltsIndex = TitanPalsAlts[TitanPalsFaction][TitanPalsRealm];
	local Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note
	local friendsAdded = 0;
	for i, v in pairs( TitanPalsAlts[TitanPalsFaction][TitanPalsRealm] ) do
		local totalPals = 0;
		local friendsIndex
		for friendsIndex=1, NumPals do
			Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note = GetFriendInfo(friendsIndex);
			if ( i == Pals_name ) then totalPals = 1; end
		end
		if ( totalPals == 0 and i ~= TitanPalsRealmUnit ) then 
			AddFriend(i); 
			friendsAdded = friendsAdded + 1; 
		end
	end
	if ( friendsAdded > 0 ) then ShowFriends(); end
end

-- **************************************************************************
-- NAME : TitanPalsCommand_Handler()
-- DESC : <research>
-- **************************************************************************
function TitanPalsCommand_Handler(events)
	local TitanPalsCommand, evn2, evn3, evn4 = TitanPalsUtils_Args(events);
	if ( not TitanPalsCommand or TitanPalsCommand == "" ) then
		TitanPalsCommand = "help";
	end

	if ( TitanPalsCommand == "fixpal") then TitanPalsUtils_FixPal(evn2, evn3, evn4)
	elseif ( TitanPalsCommand == "help" ) then TitanPalsCommand_Help(); end
end

-- **************************************************************************
-- NAME : TitanPalsCommand_Help()
-- DESC : <research>
-- **************************************************************************

function TitanPalsCommand_Help()
	TitanPalsUtils_Print("TitanPals Current Options:");
	TitanPalsUtils_Print("    /Pals fixpal <Name> <Class #> <Level>");
	TitanPalsUtils_Print("                        1 = Warrior");
	TitanPalsUtils_Print("                        2 = Mage");
	TitanPalsUtils_Print("                        3 = Warlock");
	TitanPalsUtils_Print("                        4 = Paladin");
	TitanPalsUtils_Print("                        5 = Priest");
	TitanPalsUtils_Print("                        6 = Hunter");
	TitanPalsUtils_Print("                        7 = Shaman");
	TitanPalsUtils_Print("                        8 = Rogue");
	TitanPalsUtils_Print("                        9 = Druid");
	TitanPalsUtils_Print("                        10 = Death Knight");
	TitanPalsUtils_Print("    /Pals help");
end

-- **************************************************************************
-- NAME : TitanPanelPalsTimer_Handler()
-- DESC : <research>
-- **************************************************************************
function TitanPalsTimer_Handler()
	ShowFriends();
	TitanPanelButton_UpdateButton(TITAN_PALS_ID);	
	TitanPanelButton_UpdateTooltip();
	if ( not TitanPalsVars["Version"] ) then DEFAULT_CHAT_FRAME:AddMessage("|cffc41f3bTitan[|r|cffeda55fPals|r|cffc41f3b]: There has been major changes since the last release please type|r |cffeda55f/script TitanPalsUtils_Upgrade|r")
	elseif ( TitanPalsVars["Version"] ~= TITAN_PALS_ADDVER ) then 
		DEFAULT_CHAT_FRAME:AddMessage("|cffc41f3bTitan[|r|cffeda55fPals|r|cffc41f3b]: There has been a minor script change in this release fixing Changeing version number..|r")
		TitanPalsVars["Version"] = "r3";
	end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnEvent()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPalsButton_OnEvent(self, event, ...)
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		local TPPN = UnitName("player");
		local TPPR = GetCVar("realmName");
		local TPGF = UnitFactionGroup("player");
		TitanPanelPals_Init(TPPN, TPPR, TPGF)
	end
	TitanPanelButton_UpdateButton(TITAN_PALS_ID);	
	TitanPanelButton_UpdateTooltip();
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnClick()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPalsButton_OnClick(self, button)
	if ( button == "LeftButton" ) then
		ToggleFriendsFrame(1);
	else
		TitanPanelButton_OnClick(self, button);
	end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_GetButtonText()
-- DESC : <research>
-- **************************************************************************
function TitanPanelPalsButton_GetButtonText()
	local NumIgnore = GetNumIgnores();
	local CurCounts = TitanPanelPals_GetButtonCounts();
	local NumPals, NumPalsOnline = string.match( CurCounts, '(.*):(.*)' )
	-- create string for Titan bar display
	local buttonRichText = format(TITAN_PALS_BUTTON_TEXT, TitanPalsUtils_GetColoredText("green", NumPalsOnline), TitanPalsUtils_GetColoredText("highlight", NumPals), TitanPalsUtils_GetColoredText("red", NumIgnore));
	return TITAN_PALS_BUTTON_LABEL, buttonRichText;
end

-- **************************************************************************
-- NAME : TitanPanelPals_GetButtonCounts()
-- DESC : <research>
-- **************************************************************************

function TitanPanelPals_GetButtonCounts()
	local NumPals = GetNumFriends();
	local NumPalsOnline = 0;
	local Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note
	local friendIndex

	-- get a count of the number of online friends
	for friendIndex=1, NumPals do
		Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note = GetFriendInfo(friendIndex);
		if ( Pals_connected ) then
			NumPalsOnline = NumPalsOnline + 1;
		end
	end
	return NumPals..":"..NumPalsOnline;
end

-- **************************************************************************
-- NAME : palsWisper()
-- DESC : <research>
-- **************************************************************************
function palsWisper()
	if ( not ChatFrameEditBox:IsVisible() ) then
		ChatFrame_OpenChat("/w "..this.value.." ");
	else
		ChatFrameEditBox:SetText("/w "..this.value.." ");
	end
end

-- **************************************************************************
-- NAME : TitanPals_DevDebug()
-- DESC : <research>
-- **************************************************************************
function TitanPals_DevDebug()
   TitanToggleVar(TITAN_PALS_ID, "Debug");
   TitanPanelButton_UpdateButton(TITAN_PALS_ID);
end

--------------------------------------------------------------------------
-- NAME : TitanPalsUtils_Debug();
-- DESC : </script TitanSetVar(TITAN_PALS_ID,"Debug",1)>
--------------------------------------------------------------------------
function TitanPalsUtils_Debug(func, msg)
	if ( func ) then
		if ( TitanGetVar(TITAN_PALS_ID, "Debug") ) then DEFAULT_CHAT_FRAME:AddMessage("|cffc41f3bTitan[|r|cffeda55fPals|r|cffc41f3b] Debug(|r|cffeda55f"..func.."|r|cffc41f3b):|r "..msg.."|r") end
	end
end


-- **************************************************************************
-- NAME : TitanPalsUtils_Upgrade()
-- DESC : <research>
-- **************************************************************************

function TitanPalsUtils_Upgrade()
	local db = true;
	StaticPopupDialogs["TITAN_PALS_RESET_BAR"] = {
		text = TITAN_PALS_WARNING_TEXT.."\n\n"..TITAN_PALS_RESET_WARNING,
		button1 = ACCEPT,
		button2 = CANCEL,
		OnAccept = function(self) 
			TitanPalsUtils_MasterReset(db);
		end,
		showAlert = 1,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1
	};
	StaticPopup_Show("TITAN_PALS_RESET_BAR");
end

-- **************************************************************************
-- NAME : TitanPalsUtils_Args()
-- DESC : <research>
-- **************************************************************************

function TitanPalsUtils_Args(strand)
	local list = {};
	local a,b,c,d;
	for w in string.gmatch(strand, "%S+") do 
		table.insert(list,w) 
	end
	if ( list[1] ) then a = list[1]; end
	if ( list[2] ) then b = list[2]; end
	if ( list[3] ) then c = list[3]; end
	if ( list[4] ) then d = table.concat(list," ",4); end
	return a,b,c,d;
end

-- **************************************************************************
-- NAME : TitanPalsUtils_Print()
-- DESC : <research>
-- **************************************************************************

function TitanPalsUtils_Print(toPrint)
	DEFAULT_CHAT_FRAME:AddMessage(toPrint)
end

-- **************************************************************************
-- NAME : TitanPalsUtils_MasterReset()
-- DESC : <research>
-- **************************************************************************

function TitanPalsUtils_MasterReset(db)
	if ( db ) then
		-- Clear Data of Pals that have been clocted thus far
		TitanPalsList = {}
		TitanPalsAlts = {}
		--	TitanPalsVars = {}
	end

	TitanPalsVars["Version"] = "r3";

	-- Revert To Default Values
	TitanSetVar(TITAN_PALS_ID, "ShowIcon", 1);
	TitanSetVar(TITAN_PALS_ID, "ShowLabelText", 1);
	TitanSetVar(TITAN_PALS_ID, "ShowOffline", false);
	TitanSetVar(TITAN_PALS_ID, "ShowIgnored", false);
	TitanSetVar(TITAN_PALS_ID, "ShowClass", 1);
	TitanSetVar(TITAN_PALS_ID, "ShowNote", false);
	TitanSetVar(TITAN_PALS_ID, "TitanPalsSync", false);
	TitanSetVar(TITAN_PALS_ID, "LoadMessage", 1);
	TitanSetVar(TITAN_PALS_ID, "NoAlts", 1);
	TitanSetVar(TITAN_PALS_ID, "Debug", false);

	-- Reload the UI to Save Settings
	ReloadUI()
end